package ie.ucd.csi.comp10050.cellular_automata;

/**
 * A cell whose value is one of a specific set of values.
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 */

public class IndexedCellKey extends IndexedCell {
  /**
   * The value of this indexed cell.
   */
  //@ private invariant validValue(my_value);
  private long my_value;

  /**
   * The maximum index value for indexed cells.
   */
  //@ private invariant validValue(my_indexsize);
  private int my_indexsize;
  
  private boolean use_unicode, use_digits, use_density;
  //@ private invariant ((use_unicode ? 1 : 0) ^ (use_digits ? 1 : 0) ^ (use_density ? 1 : 0)) == 1;

  // @bon query

  /** {@inheritDoc} */
  public final boolean validValue(final long the_value) {
    return (the_value <= my_indexsize);
  }

  /** {@inheritDoc} */
  public final long validValueCount() {
    return my_indexsize + 1;
  }

  /** {@inheritDoc} */
  public final char textualRepresentation() {
    if (use_unicode)
      return (char)my_value;
    else if (use_digits)
      return (char)(my_value/validValueCount() + '0');
    else if (use_density) {
      int density = (int)(my_value/validValueCount() % 11);
      switch (density) {
        case 0: return ' ';
        case 1: return '\'';
        case 2: return '`';
        case 3: return '.';
        case 4: return ',';
        case 5: return '~';
        case 6: return '-';
        case 7: return '+';
        case 8: return '*';
        case 9: return '%';
        case 10: return '@';
        case 11: return '#';
        default: assert false; //@ assert false;
                 return ' ';
      }
    } else {
    	assert false; //@ assert false;
    	return ' ';
    }
  }

  /** {@inheritDoc} */
  public final long value() {
    return my_value;
  }

  /** {@inheritDoc} */
  public final boolean equals(final /*@ nullable */ Object a_cell) {
    final IndexedCellKey othercell = (IndexedCellKey)a_cell;
    return (othercell.my_value == this.my_value & othercell.indexSize() == this.indexSize());
  }

  /** {@inheritDoc} */
  public final int indexSize() {
    return my_indexsize;
  }

  // @bon command

  /** {@inheritDoc} */
  public final void setValue(final long the_value) {
    my_value = the_value;
    return;
  }

  /** {@inheritDoc} */
  public final void setIndexSize(final int an_indexsize) {
    my_indexsize = an_indexsize;
    return;
  }

  // @bon constraint
}
