package ie.ucd.csi.comp10050.cellular_automata;

/**
 * A cell whose values are any long whatsoever.
 *
 * <p> This cell is used to model intensities. </p>
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 */

public abstract class IntensityCell extends IndexedCell {
  // @bon query

  /** {@inheritDoc} */
  //@ also
  //@ ensures \result <==> ((0 <= the_value) & (the_value != Integer.MAX_VALUE));
  public abstract /*@ pure @*/ boolean validValue(final long the_value);

  /** {@inheritDoc} */
  // @constraint The values of an intensity cell range from zero to
  // the maximal representable value.
  //@ also
  //@ ensures \result == Long.MAX_VALUE;
  public abstract long validValueCount();

  /**
   * @return the value of this cell as a percentage.
   */
  //@ ensures \result == ((float)value())/Integer.MAX_VALUE*100.0;
  public abstract float valueAsPercentage();

  /**
   * @param a_set_of_values the set of values from which we derive a new value.
   * @return derive a new value from a set of existing values.
   */
  public abstract long deriveNewValue(final /*@ non_null @*/ long[] a_set_of_values);

  // @bon command

  // @bon constraint
}
