/**
 * @author Hermann Lehner, Aleksy Schubert
 *
 * The Bill class provides an abstract implementation of the bill
 * functionality. It calculates the aggregate cost for series of
 * investments based on the implementation of the method which gives 
 * the cost of a single round (to be implemented in subclasses).
 * 
 */
abstract class OriginalBill {
  
  private int sum;
  //@ invariant sum>=0;
 
  /**
   * This method gives a cost of a single round.
   * 
   * @param x is the number of the particular round
   * @return the cost of the investment in this round, it's not
   *         greater than <code>x</code> 
   */ 
  //@ensures \result <= x;
  abstract int round_cost(int x) throws Exception;
  
  /**
   * This method calculates the cost of the whole series of investments.
   *
   * @return <code>true</code> when the calculation is successful and
   *         <code>false</code> when the calculation cannot be performed
   */
  //@requires n > 0;
  //@ensures sum <=  \old(sum)+n*(n+1)/2;
  public boolean produce_bill(int n){
    int i;
    try{
      //@ loop_invariant i <= n + 1 && sum <=  \old(sum)+(i+1)*i/2;
      for (i=1;i<=n;i++) 
      {
        this.sum = this.sum + round_cost(i);
      }
      return true;
    } catch (Exception e){
      return false;
    }
  }
}
